if SERVER then
	AddCSLuaFile("pills_animations.lua")
end

function random_sequence(...)
	local t = {}
	t.random = true
	t.sequences = arg
	return t
end

PillClass.animation = nil
PillClass.animations = {
	crouch = nil,
	crouchWalk = nil,
	idle = nil,
	jump = nil,
	run = nil,
	swim = nil,
	walk = nil
	}
PillClass.forced = nil

// called each frame to make the animations play
function PillClass:Animate()
	local sequence, rate
	if self.forced and CurTime() < self.forced.time then
		sequence, rate = self.forced.sequence, self.forced.rate
	else
		self.forced = nil
		sequence, rate = self:SequenceForAnimation(self:ChooseAnimation())
	end
	if sequence then
		local sequenceIndex = self.player:LookupSequence(sequence)
		if self.player:GetSequence() != sequenceIndex then
			self.player:ResetSequence(sequenceIndex)
		end
		self.player:SetPlaybackRate(rate)
		if self.ghost then
			sequenceIndex = self.ghost:LookupSequence(sequence)
			if self.ghost:GetSequence() != sequenceIndex then
				self.ghost:Fire("setanimation", sequence, 0)
			end
			self.ghost:SetPlaybackRate(rate)
		end
	end
end

// returns the animation that should be playing
function PillClass:ChooseAnimation()
	local speed = self.player:GetVelocity():Length()
	if self.player:OnGround() then
		if self.player:Crouching() and self.animations.crouch or self.animations.crouchWalk then
			if self.animations.crouchWalk and speed > 0 then
				return "crouchWalk"
			elseif self.animations.crouch then
				return "crouch"
			end
		elseif self.animations.run and not self.player:Crouching() and self.player:KeyDown(IN_SPEED) then
			return "run"
		elseif self.animations.walk and speed > 0 then
			return "walk"
		end
	else
		if self.animations.swim and self.waterlevel and self.player:WaterLevel() >= self.waterlevel then
			return "swim"
		elseif self.animations.jump then
			return "jump"
		end
	end
	return "idle"
end

// forces an animation to play for the duration specified
function PillClass:ForceAnimation(animation, duration)
	self.forced = nil
	local s, r = self:SequenceForAnimation(animation)
	self:ForceSequence(s, r, duration)
end

// forces a mdl sequence to play for the duration and at the rate specified
function PillClass:ForceSequence(sequence, rate, duration)
	self.forced = {}
	self.forced.sequence = sequence
	self.forced.rate = rate
	self.forced.time = CurTime() + duration
end

// turns an animation into a mdl sequence
function PillClass:SequenceForAnimation(animation)
	local sequence, rate
	if type(self.animations[animation]) == "table" then
		if self.animations[animation].random then
			local a = self.animations[animation].sequences
			return a[math.random(#a)], self.animations[animation].rate or 1
		else
			return self.animations[animation][1], self.animations[animation][2]
		end
	else
		return self.animations[animation], 1
	end
end
